"""
author: deadc0de6 (https://github.com/deadc0de6)
Copyright (c) 2017, deadc0de6
basic unittest for the import function
"""


import unittest
import dotdrop


class TestDummy(unittest.TestCase):

    dotdrop.main()


def main():
    unittest.main()


if __name__ == '__main__':
    main()
